@@ -3,13 +3,14 @@
 import datetime
 import h5py
 import logging
+import sys
 from mpi4py import MPI
 import numpy as np
 import pmesh.pm as pmesh
 import warnings
 from .configure_runtime import configure_runtime
-from .hamiltonian import DefaultNoChi, DefaultWithChi, SquaredPhi
-from .input_parser import check_config
+from .hamiltonian import get_hamiltonian
+from .input_parser import check_config, check_charges
 from .logger import Logger, format_timedelta
 from .file_io import distribute_input, OutDataset, store_static, store_data
 from .field import (compute_field_force, update_field,
@@ -34,7 +35,7 @@ def main():
     if rank == 0:
         start_time = datetime.datetime.now()
 
-    args, config = configure_runtime(comm)
+    args, config, prng = configure_runtime(sys.argv[1:], comm)
 
     if args.double_precision:
         dtype = np.float64
@@ -95,13 +96,13 @@ def main():
         else:
             charges_flag = False
 
+    if charges_flag:
+        check_charges(charges, comm=comm)
+
     config = check_config(config, indices, names, types, comm=comm)
-    if config.n_print:
-        if config.n_flush is None:
-            config.n_flush = 10000 // config.n_print
 
     if config.start_temperature:
-        velocities = generate_initial_velocities(velocities, config, comm=comm)
+        velocities = generate_initial_velocities(velocities, config, prng, comm=comm)
     elif config.cancel_com_momentum:
         velocities = cancel_com_momentum(velocities, config, comm=comm)
 
@@ -146,22 +147,7 @@ def main():
             dtype="f4" if dtype == np.float32 else np.float64, comm=comm
         )
 
-    if config.hamiltonian.lower() == "defaultnochi":
-        hamiltonian = DefaultNoChi(config)
-    elif config.hamiltonian.lower() == "defaultwithchi":
-        hamiltonian = DefaultWithChi(
-            config, config.unique_names, config.type_to_name_map
-        )
-    elif config.hamiltonian.lower() == "squaredphi":
-        hamiltonian = SquaredPhi(config)
-    else:
-        err_str = (
-            f"The specified Hamiltonian {config.hamiltonian} was not "
-            f"recognized as a valid Hamiltonian."
-        )
-        Logger.rank0.log(logging.ERROR, err_str)
-        if rank == 0:
-            raise NotImplementedError(err_str)
+    hamiltonian = get_hamiltonian(config)
 
     Logger.rank0.log(logging.INFO, f"pfft-python processor mesh: {str(pm.np)}")
 
@@ -202,33 +188,23 @@ def main():
         names,
         types,
     ]
-    args_recv = [
-        "positions",
-        "velocities",
-        "indices",
-        "bond_forces",
-        "angle_forces",
-        "dihedral_forces",
-        "reconstructed_forces",
-        "field_forces",
-        "names",
-        "types",
-    ]
 
     if charges_flag:
         args_in.append(charges)
         args_in.append(elec_forces)
-        args_recv.append("charges")
-        args_recv.append("elec_forces")
-    if molecules_flag:
-        args_recv.append("bonds")
-        args_recv.append("molecules")
-
-    _str_receive_dd = ",".join(args_recv)
-    _cmd_receive_dd = f"({_str_receive_dd}) = dd"
 
     if config.domain_decomposition:
-        dd = domain_decomposition(
+        (positions,
+        velocities,
+        indices,
+        bond_forces,
+        angle_forces,
+        dihedral_forces,
+        reconstructed_forces,
+        field_forces,
+        names,
+        types,
+        *optional) = domain_decomposition(
             positions,
             pm,
             *tuple(args_in),
@@ -238,7 +214,29 @@ def main():
             comm=comm,
         )
 
-        exec(_cmd_receive_dd)
+        if charges_flag:
+            charges = optional.pop(0)
+            elec_forces = optional.pop(0)
+        if molecules_flag:
+            bonds = optional.pop(0)
+            molecules = optional.pop(0)
+
+        # rebuild args_in to point to correct arrays
+        args_in = [
+            velocities,
+            indices,
+            bond_forces,
+            angle_forces,
+            dihedral_forces,
+            reconstructed_forces,
+            field_forces,
+            names,
+            types,
+        ]
+
+        if charges_flag:
+            args_in.append(charges)
+            args_in.append(elec_forces)
 
     if not args.disable_field:
         layouts = [pm.decompose(positions[types == t]) for t in range(config.n_types)]  # noqa: E501
@@ -428,6 +426,8 @@ def main():
             field_forces + bond_forces + angle_forces + dihedral_forces
             + reconstructed_forces
         )
+        if charges_flag:
+            forces_out += elec_forces
         store_data(
             out_dataset, step, frame, indices, positions, velocities,
             forces_out, config.box_size, temperature, kinetic_energy,
@@ -443,10 +443,10 @@ def main():
         last_step_time = datetime.datetime.now()
 
     # MD loop
-    for step in range(config.n_steps):
+    for step in range(1, config.n_steps + 1):
         current_step_time = datetime.datetime.now()
 
-        if step == 0 and args.verbose > 1:
+        if step == 1 and args.verbose > 1:
             Logger.rank0.log(logging.INFO, f"MD step = {step:10d}")
         else:
             log_step = False
@@ -622,16 +622,50 @@ def main():
             if not args.disable_dihedrals:
                 dihedral_energy = comm.allreduce(dihedral_energy_, MPI.SUM)
 
-        if step != 0 and config.domain_decomposition:
+        if config.domain_decomposition:
             if np.mod(step, config.domain_decomposition) == 0:
                 positions = np.ascontiguousarray(positions)
                 bond_forces = np.ascontiguousarray(bond_forces)
                 angle_forces = np.ascontiguousarray(angle_forces)
                 dihedral_forces = np.ascontiguousarray(dihedral_forces)
 
+                (positions,
+                velocities,
+                indices,
+                bond_forces,
+                angle_forces,
+                dihedral_forces,
+                reconstructed_forces,
+                field_forces,
+                names,
+                types,
+                *optional) = domain_decomposition(
+                    positions,
+                    pm,
+                    *tuple(args_in),
+                    molecules=molecules if molecules_flag else None,
+                    bonds=bonds if molecules_flag else None,
+                    verbose=args.verbose,
+                    comm=comm,
+                )
+
+                if charges_flag:
+                    charges = optional.pop(0)
+                    elec_forces = optional.pop(0)
+                if molecules_flag:
+                    bonds = optional.pop(0)
+                    molecules = optional.pop(0)
+
+                # rebuild args_in to point to correct arrays
                 args_in = [
-                    velocities, indices, bond_forces, angle_forces,
-                    dihedral_forces, reconstructed_forces, field_forces, names,
+                    velocities,
+                    indices,
+                    bond_forces,
+                    angle_forces,
+                    dihedral_forces,
+                    reconstructed_forces,
+                    field_forces,
+                    names,
                     types,
                 ]
 
@@ -639,14 +673,6 @@ def main():
                     args_in.append(charges)
                     args_in.append(elec_forces)
 
-                dd = domain_decomposition(  # noqa: F841
-                    positions, pm, *tuple(args_in),
-                    molecules=molecules if molecules_flag else None,
-                    bonds=bonds if molecules_flag else None,
-                    verbose=args.verbose, comm=comm,
-                )
-                exec(_cmd_receive_dd)
-
                 positions = np.asfortranarray(positions)
                 bond_forces = np.asfortranarray(bond_forces)
                 angle_forces = np.asfortranarray(angle_forces)
@@ -717,7 +743,7 @@ def main():
 
         # Thermostat
         if config.target_temperature:
-            csvr_thermostat(velocities, names, config, comm=comm)
+            csvr_thermostat(velocities, names, config, prng, comm=comm)
 
         # Remove total linear momentum
         if config.cancel_com_momentum:
@@ -726,7 +752,7 @@ def main():
 
         # Print trajectory
         if config.n_print > 0:
-            if np.mod(step, config.n_print) == 0 and step != 0:
+            if np.mod(step, config.n_print) == 0:
                 frame = step // config.n_print
                 if not args.disable_field:
                     (
@@ -756,6 +782,8 @@ def main():
                     field_forces + bond_forces + angle_forces
                     + dihedral_forces + reconstructed_forces
                 )
+                if charges_flag:
+                    forces_out += elec_forces
                 store_data(
                     out_dataset, step, frame, indices, positions, velocities,
                     forces_out, config.box_size, temperature, kinetic_energy,
@@ -784,7 +812,7 @@ def main():
             ),
         )
 
-    if config.n_print > 0 and np.mod(config.n_steps - 1, config.n_print) != 0:
+    if config.n_print > 0 and np.mod(config.n_steps, config.n_print) != 0:
         if not args.disable_field:
             update_field(
                 phi, layouts, force_on_grid, hamiltonian, pm, positions, types,
@@ -823,6 +851,8 @@ def main():
             field_forces + bond_forces + angle_forces + dihedral_forces
             + reconstructed_forces
         )
+        if charges_flag:
+            forces_out += elec_forces
         store_data(
             out_dataset, step, frame, indices, positions, velocities,
             forces_out, config.box_size, temperature, kinetic_energy,
